import java.awt.*; import javax.swing.*; import java.net.URL; import java.awt.image.*; 
import java.io.*; 

/*
	Lib_Image.java 
	(c)1999-2002 SSC Development Team
	Function: Managing basic image manipulation and retrieval functions.
	Allowed Dependencies: None	
*/

public class Lib_Image 
{
	public static Image getImage(String filename)
	{
	return Toolkit.getDefaultToolkit().createImage(filename);
	}
	
	public static Image getImage(ImageProducer fis)
	{
	return Toolkit.getDefaultToolkit().createImage(fis);
	}
	
	public static BufferedImage cropImage(Image source, int x, int y, int w, int h) 
	{
    //I commented out the deprecated version. Code might still be handy for other stuff though
    //ImageFilter         filter = new CropImageFilter(x, y, w, h);
    //FilteredImageSource fis    = new FilteredImageSource(source.getSource(),filter);
    //return getImage(fis); 
    BufferedImage source2 = toBufferedImage(source);
    return(source2.getSubimage(x,y,w,h)); 
    }


	// Converts Image to BufferedImage
    public static BufferedImage toBufferedImage(Image image) {
        if (image instanceof BufferedImage) { return (BufferedImage)image; }
    
        // This code ensures that all the pixels in the image are loaded
        image = new ImageIcon(image).getImage();
    
        // Determine if the image has transparent pixels; for this method's
        // implementation, see e665 Determining If an Image Has Transparent Pixels
        boolean hasAlpha = hasAlpha(image);
    
        // Create a buffered image with a format that's compatible with the screen
        BufferedImage bimage = null;
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        try {
            // Determine the type of transparency of the new buffered image
            int transparency = Transparency.OPAQUE;
            if (hasAlpha) {
                transparency = Transparency.BITMASK;
            }
    
            // Create the buffered image
            GraphicsDevice gs = ge.getDefaultScreenDevice();
            GraphicsConfiguration gc = gs.getDefaultConfiguration();
            bimage = gc.createCompatibleImage(
                image.getWidth(null), image.getHeight(null), transparency);
        } catch (HeadlessException e) {
            // The system does not have a screen
        }
    
        if (bimage == null) {
            // Create a buffered image using the default color model
            int type = BufferedImage.TYPE_INT_RGB;
            if (hasAlpha) {
                type = BufferedImage.TYPE_INT_ARGB;
            }
            bimage = new BufferedImage(image.getWidth(null), image.getHeight(null), type);
        }
    
        // Copy image to buffered image
        Graphics g = bimage.createGraphics();
    
        // Paint the image onto the buffered image
        g.drawImage(image, 0, 0, null);
        g.dispose();
    
        return bimage;
    }

    // This method returns true if the specified image has transparent pixels
    public static boolean hasAlpha(Image image) {
        // If buffered image, the color model is readily available
        if (image instanceof BufferedImage) {
            BufferedImage bimage = (BufferedImage)image;
            return bimage.getColorModel().hasAlpha();
        }
    
        // Use a pixel grabber to retrieve the image's color model;
        // grabbing a single pixel is usually sufficient
         PixelGrabber pg = new PixelGrabber(image, 0, 0, 1, 1, false);
        try {
            pg.grabPixels();
        } catch (InterruptedException e) {
        }
    
        // Get the image's color model
        ColorModel cm = pg.getColorModel();
        return cm.hasAlpha();
    }

  	/* Creates an ImageIcon. */
    protected static ImageIcon getIcon(String path) {
	    return new ImageIcon(path);
    }
    
    
    //TODO: Implement!
    //public static void renderSprites(Entities ent, Graphics g, JPanel m, int x, int y, int w, int h)
	    
	public static void renderMap(WalkMap map, Graphics g, JPanel m, int x, int y, int w, int h, int layer) 
	{ 
		int ini_x = x;
		int ini_y = y;
		
		int width = map.getWidth();
		int height = map.getHeight();
	
		if(ini_x<0) ini_x=0;
		if(ini_y<0) ini_y=0;
		
		int max_x = x+w;
		int max_y = y+h;
	
		int max_tx = (ini_x+w)/24;
		int max_ty = (ini_y+h)/24;
	
		if(max_tx >= width) {
			max_tx = width-1;
			max_x = width*24-1;
			ini_x = max_x-w;
			if(ini_x<0) {
				ini_x = 0;
				}		
			}
		if(max_ty >= height) {
			max_ty = height-1;
			max_y = height*24-1;
			ini_y = max_y-h;
			if(ini_y<0) {
				ini_y = 0;
				}
			}	
		
		if(layer==0) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
					g.drawImage(map.getBlock(i,j,0), 24*i-ini_x, 24*j-ini_y, m);
				}
			}
		}	
		
		if(layer==1) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
					if(map.getValue(i,j,1)>0) {
					g.drawImage(map.getBlock(i,j,1), 24*i-ini_x, 24*j-ini_y, m); }
				}
			}
		}	
		
		if(layer==2) {
			for(int i=ini_x/24;i<=max_tx;i++) {
			for(int j=ini_y/24;j<=max_ty;j++) {
					if(map.getValue(i,j,2)>0) {
					g.drawImage(map.getBlock(i,j,2), 24*i-ini_x, 24*j-ini_y, m); }
				}
			}
		}			
		
	}    
	
}